"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createResetResources = exports.createFetchResource = void 0;

var _creators = require("./creators");

var _repository = require("./repository");

var _resource = require("./resource");

/**
 * Create action creator function to fetch resource.
 *
 * @param {string} resourceName Resource name, used to distinguish different resources.
 * @param {string} id Resource ID.
 * @param {Function} repositoryExtractor Callback consuming state as a parameter and returning the repository object.
 * @param {Function} fetchFunction Callback implementing fetch algorithm and calling the first parameter passed with the
 *                                 result or the second with an error.
 * @param {object} [options={}] Options.
 * @returns {Function} Function to be used as an action creator.
 */
var createFetchResource = function createFetchResource(resourceName, id, repositoryExtractor, fetchFunction) {
  var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
  return function (dispatch, getState) {
    var repository = repositoryExtractor(getState());
    var resourceInRepository = (0, _repository.getResourceById)(repository, id);

    if (resourceInRepository) {
      if ((0, _resource.isReceived)(resourceInRepository) && options.ttl && !(0, _resource.isExpired)(resourceInRepository, options.ttl)) {
        if (!options.silentAlready) {
          dispatch((0, _creators.fetchResourceAlreadyReceived)(resourceName, id));
        }

        return;
      }

      if ((0, _resource.isRequested)(resourceInRepository)) {
        if (!options.silentAlready) {
          dispatch((0, _creators.fetchResourceAlreadyRequested)(resourceName, id));
        }

        return;
      }
    }

    dispatch((0, _creators.fetchResourceRequested)(resourceName, id));

    var dispatchReceived = function dispatchReceived(data) {
      dispatch((0, _creators.fetchResourceReceived)(resourceName, id, data));
    };

    var dispatchFailed = function dispatchFailed(error) {
      dispatch((0, _creators.fetchResourceFailed)(resourceName, id, error));
    };

    fetchFunction(dispatchReceived, dispatchFailed);
  };
};
/**
 * Create action creator function to reset resources.
 *
 * @param {string} resourceName Resource name, used to distinguish different resources.
 * @returns {Function} Function to be used as an action creator.
 */


exports.createFetchResource = createFetchResource;

var createResetResources = function createResetResources(resourceName) {
  return function (dispatch) {
    dispatch((0, _creators.resourcesReset)(resourceName));
  };
};

exports.createResetResources = createResetResources;